import re
from typing import Dict, List

import romkan

_BOIN = ['a', 'i', 'u', 'e', 'o']

_LOWER_PATTERN = re.compile('[a-z]')


def parse_skkdict(path: str, encoding: str = 'euc-jp'):
    nasi: Dict[str, List[str]] = {}
    ari: Dict[str, List[str]] = {}
    target = ari

    with open(path, 'r', encoding=encoding) as fp:
        for line in fp:
            if line == ";; okuri-ari entries.\n":
                target = ari
            if line == ";; okuri-nasi entries.\n":
                target = nasi
            if line.startswith(';;'):
                continue

            m = line.strip().split(' ', 1)
            if len(m) == 2:
                yomi: str = m[0]
                kanjis: List[str] = m[1].lstrip('/').rstrip('/').split('/')
                kanjis = [re.sub(';.*', '', k) for k in kanjis]

                target[yomi] = kanjis

    return ari, nasi


def write_skkdict(outfname: str, dict_ari, dict_nasi, encoding: str = 'utf-8'):
    with open(outfname, 'w', encoding=encoding) as ofh:
        ofh.write(";; okuri-ari entries.\n")
        for yomi in sorted(dict_ari.keys()):
            kanjis = dict_ari[yomi]
            if len(kanjis) != 0:
                ofh.write("%s /%s/\n" % (yomi, '/'.join(kanjis)))

        ofh.write(";; okuri-nasi entries.\n")
        for yomi in sorted(dict_nasi.keys()):
            kanjis = dict_nasi[yomi]
            if len(kanjis) != 0:
                ofh.write("%s /%s/\n" % (yomi, '/'.join(kanjis)))


def merge_skkdict(dicts: List[Dict[str, List[str]]]) -> Dict[str, List[str]]:
    result = {}

    for dic in dicts:
        for kana, kanjis in dic.items():
            if kana not in result:
                result[kana] = []
            for kanji in kanjis:
                if kanji not in result[kana]:
                    result[kana].append(kanji)

    return result


def expand_okuri(kana: str, kanjis: List[str]):
    if kana[-1].isalpha():
        if kana[-1] in _BOIN:
            okuri = romkan.to_hiragana(kana[-1])
            yield kana[:-1] + okuri, [kanji + okuri for kanji in kanjis]
        else:
            for b in _BOIN:
                okuri = romkan.to_hiragana(kana[-1] + b)
                if _LOWER_PATTERN.match(okuri):
                    # wu のように、変換できないものは無視する。
                    continue
                yield kana[:-1] + okuri, [kanji + okuri for kanji in kanjis]
    else:
        yield kana, kanjis


def ari2nasi(src):
    retval = {}
    for kana, kanjis in src.items():
        for kkk, vvv in expand_okuri(kana, kanjis):
            retval[kkk] = vvv
    return retval



from . import skkdictutils

__all__ = ['parse_skkdict', 'merge_skkdict', 'ari2nasi', 'write_skkdict', 'expand_okuri']

__version__ = '0.1.0'

parse_skkdict = skkdictutils.parse_skkdict
merge_skkdict = skkdictutils.merge_skkdict
ari2nasi = skkdictutils.ari2nasi
write_skkdict = skkdictutils.write_skkdict
expand_okuri = skkdictutils.expand_okuri



import setuptools
import re
import os

with open(os.path.join('skkdictutils', '__init__.py'), 'r', encoding='utf8') as f:
    version = re.compile(
        r".*__version__ = '(.*?)'", re.S).match(f.read()).group(1)

setuptools.setup(
    name="skkdictutils",
    install_requires=['romkan>=0.2.1'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    extras_require={
    },
    entry_points={
    },
    packages=setuptools.find_packages(),
    version=version,
    license='MIT License',
    platforms=['POSIX', 'Windows', 'Unix', 'MacOS'],
    description = 'SKK dictionary utility functions',
    author='tokuhirom',
    author_email='tokuhirom@gmail.com',
    url='https://github.com/tokuhirom/skkdictutils-py',
    keywords = ['Japanese'],
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        "Programming Language :: Python :: 3",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Natural Language :: Japanese',
        'Topic :: Text Processing',
    ],
    python_requires='>=3.6',
    long_description='%s' % (open('README.md', encoding='utf8').read()),
    test_suite = 'pytest',
)
